package org.LiveGraph.bootstrap;


import static org.LiveGraph.bootstrap.UpgradeManager.AutoCheckFrequency.*;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.SocketTimeoutException;
import java.net.URL;
import java.net.URLConnection;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Properties;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.WindowConstants;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;

import org.LiveGraph.LiveGraph;
import org.LiveGraph.gui.GUIManager;
import org.LiveGraph.gui.Tools;

import com.softnetConsult.utils.exceptions.Bug;
import com.softnetConsult.utils.exceptions.UnexpectedSwitchCase;
import com.softnetConsult.utils.files.FileTools;
import com.softnetConsult.utils.mutableWrappers.MutableInt;
import com.softnetConsult.utils.sys.SystemTools;

public class UpgradeManager {

public static final String upgradeOptionsFileName = "autoUpdate.lguds";

public enum AutoCheckFrequency {

	DEFAULT(1),
	DAILY(1),
	WEEKLY(7),
	MONTHLY(30),
	MANUAL_ONLY(Integer.MAX_VALUE);
	
	private int freq = 1;
	private AutoCheckFrequency(int freq) { this.freq = freq; }
	public int getFrequency()			 { return freq; }
}

private AutoCheckFrequency frequency = DEFAULT;
private long lastCheckTS = 0L;
private volatile boolean checkRunning = false;
private volatile boolean checkAbort = false;

public void autoUpdate() {
	
	if (!LiveGraph.application().standalone())
		return;
	
	loadSettings();
	
	if (MANUAL_ONLY == frequency)
		return;
	
	Calendar now = Calendar.getInstance();
	Calendar last = Calendar.getInstance();
	last.setTimeInMillis(lastCheckTS);
	
	last.add(Calendar.DATE, frequency.getFrequency());
	if (last.after(now))
		return;
	
	final Object[] options = new Object[] { "Check now", "Remind me later", "Auto-update options..."};
	Object sel = JOptionPane.showOptionDialog(
					null,
					"Would you like to allow LiveGraph to check whether a software update is available?",
					"Auto-Update",
					JOptionPane.DEFAULT_OPTION,
					JOptionPane.QUESTION_MESSAGE,
					null,
					options,
					options[0]);
	
	if (null == sel)
		return;
	
	if (! (sel instanceof Integer)) {
		throw new Bug("Unecperted type: " + sel.getClass().getName());
	} else {
		int opt = ((Integer) sel).intValue();
		switch(opt) {
			case 0:
				checkForUpdates(true);
				break;
				
			case 1:
				break;
				
			case 2:
				upgradeOptionsDialog();
				break;
			
			default:
				throw new UnexpectedSwitchCase(opt);
		}
	}
}

private String formatUpdateCheckURL(boolean automatic) {
	final String template = "http://live-graph.org/update-check/?version=%s&requestType=%s";
	return String.format(template,
						 LiveGraph.version,
						 automatic ? frequency.name() : "UserInitiated");
}

private void checkForUpdates(final boolean automatic) {
	
	Runnable checker = new Runnable() {
		public void run() {
			doCheckForUpdates(automatic);
		}
	};
	
	Thread worker = new Thread(checker, "LiveGRaph Software Update Checker");
	worker.start();
}

private synchronized void doCheckForUpdates(boolean automatic) {
	
	// Create dialog window:
	final JDialog dialog = new JDialog((JDialog) null, "LiveGraph auto update", false);
	dialog.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	dialog.getContentPane().setLayout(new BorderLayout());
	
	// Create close button:
	JPanel panel = new JPanel(new FlowLayout(FlowLayout.CENTER));
	final JButton closeButton = new JButton("Close");
	closeButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			closeButton.setText("Closing...");
			closeButton.setEnabled(false);
			if (checkRunning)	checkAbort = true;
			else				dialog.dispose();
        }
	});
	panel.add(closeButton, BorderLayout.CENTER);
	dialog.getContentPane().add(panel, BorderLayout.SOUTH);

	// Create text pane:
	JTextPane textPane = new JTextPane();
	textPane.setEditable(false);
	Document doc = textPane.getDocument();
	
	JScrollPane scrollPane = new JScrollPane();
	scrollPane.getViewport().add(textPane);
	dialog.getContentPane().add(scrollPane);
	dialog.pack();
	
	// Set dialog size and display:
	final int WIDTH = 620;
	final int HEIGHT = 600;
	Dimension scr = Toolkit.getDefaultToolkit().getScreenSize();
	dialog.setBounds((scr.width - WIDTH) / 2, (scr.height - HEIGHT) / 2, WIDTH, HEIGHT);
	dialog.addWindowListener(new WindowAdapter() {
		@Override public void windowClosing(WindowEvent e) {
			closeButton.setText("Closing...");
			closeButton.setEnabled(false);
			if (checkRunning)	checkAbort = true;
			else				dialog.dispose();
		}
	});
	checkRunning = true;
	dialog.setVisible(true);
	SystemTools.sleep(100);
	
	// Check for update availability:
	final String urlStr = formatUpdateCheckURL(automatic);
	try {
		URL url = new URL(urlStr);
		URLConnection conn = url.openConnection();
		conn.setConnectTimeout(1000);
		conn.setReadTimeout(10000);
		InputStream ins = null;
		try {
			doc.insertString(doc.getLength(), "\nAttempting to connect to server... ", null);
			dialog.repaint();
			ins = conn.getInputStream();
			doc.insertString(doc.getLength(), "Connected. Loading info...", null);
		} catch(SocketTimeoutException e) {
			doc.insertString(doc.getLength(), "Could not connect. Will retry shortly.", null);
			ins = null;
		}
		
		for (int attempt = 1; !checkAbort && null == ins && attempt <= 3; attempt++) {
			SystemTools.sleep(5000);
			if (checkAbort)
				break;
			url = new URL(urlStr);
			conn = url.openConnection();
			conn.setConnectTimeout(10000);
			conn.setReadTimeout(10000);
			doc.insertString(doc.getLength(), "\nReconnection attempt " + attempt + " of 3."
											+ " Attempting to connect to server... ", null);
			try {
				conn.connect();
				ins = conn.getInputStream();
				doc.insertString(doc.getLength(), "Connected. Loading info...", null);
			} catch(SocketTimeoutException e) {
				ins = null;
				doc.insertString(doc.getLength(), "Could not connect. Will retry shortly.", null);
			}
		}
		
		if (null == ins) {
			doc.insertString(doc.getLength(), "\n\nCould not connect to server.", null);
			doc.insertString(doc.getLength(), "\nVisit http://www.live-graph.org/downloads.html to"
											+ " check whether you have the latest version.", null);
		} else if (!checkAbort) {
			try {
				try {
					String oldBtnTxt = closeButton.getText();
					boolean oldBtnEnbl = closeButton.isEnabled();
					closeButton.setEnabled(false);
					closeButton.setText("READING...");
					
					textPane.setContentType("text/html");
					textPane.read(ins, textPane.getEditorKit().createDefaultDocument());
					
					closeButton.setText(oldBtnTxt);
					closeButton.setEnabled(oldBtnEnbl);
					
					lastCheckTS = System.currentTimeMillis();
					saveSettings();
				} finally {
					ins.close();
				}
			} catch(SocketTimeoutException e) {
				doc = textPane.getDocument();
				doc.insertString(doc.getLength(), "\n\nCould not read from the server.", null);
				doc.insertString(doc.getLength(), "\nVisit http://www.live-graph.org/downloads.html to"
												+ " check whether you have the latest version.", null);
			}
		}
	} catch (IOException ioe) {
		dialog.dispose();
		// Exceptions other than handled above can be treated by the default exception handler:
		throw new RuntimeException(ioe);
	} catch (BadLocationException ble) {
		dialog.dispose();
		throw new Bug("This should never happen!", ble);
	}
	
	checkRunning = false;
	if (checkAbort)
		dialog.dispose();
}

public synchronized void upgradeOptionsDialog() {
	
	if (!LiveGraph.application().standalone()) {
		JOptionPane.showMessageDialog(
				null,
				"Automatic update settings are not available since LiveGraph is not running is stand-alone mode.",
				"LiveGraph is not running in stand-alone mode",
				JOptionPane.INFORMATION_MESSAGE);
		return;
	}
	
	loadSettings();
	
	// Create dialog window:
	final JDialog dialog = new JDialog((JDialog) null, "LiveGraph auto update settings", true);
	dialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
	dialog.getContentPane().setLayout(new GridBagLayout());
	dialog.addWindowListener(new WindowAdapter() {
		@Override public void windowClosing(WindowEvent e) {
			dialog.dispose();
		}
	});
	
	// Last update info:
	dialog.getContentPane().add(new JLabel("Last update check:"), Tools.createGridBagConstraints(0, 0, 1, 1));
	String lastCheckStr = "never";
	if (0L < lastCheckTS) {
		DateFormat format = new SimpleDateFormat("dd/MM/yyyy, HH:mm:ss");
		Calendar cal = Calendar.getInstance();
		cal.setTimeInMillis(lastCheckTS);
		lastCheckStr = format.format(cal.getTime());
	}
	dialog.getContentPane().add(new JLabel(lastCheckStr), Tools.createGridBagConstraints(1, 0, 1, 1));
	
	// Update chooser:
	dialog.getContentPane().add(new JLabel("Auto-update frequency:"), Tools.createGridBagConstraints(0, 1, 1, 1));
	final JComboBox freqCombo = new JComboBox();
	int i = 0, selected = -1;
	for (AutoCheckFrequency f : AutoCheckFrequency.values()) {
		if (DEFAULT == f)
			continue;
		freqCombo.addItem(f.toString());
		if (DEFAULT == frequency && DAILY == f)
			selected = i;
		if (DEFAULT != frequency && f == frequency)
			selected = i;
		i++;
	}
	freqCombo.setSelectedIndex(selected);
	dialog.getContentPane().add(freqCombo, Tools.createGridBagConstraints(1, 1, 1, 1));
	
	// Spacer:
	dialog.getContentPane().add(Box.createVerticalStrut(10), Tools.createGridBagConstraints(0, 2, 2, 1));
	
	// Buttons:
	JPanel panel = new JPanel(new FlowLayout(FlowLayout.CENTER));
	dialog.getContentPane().add(panel, Tools.createGridBagConstraints(0, 3, 2, 1));
	
	JButton button = new JButton("Save");
	button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			String sel = (null == freqCombo.getSelectedItem() ? "" : freqCombo.getSelectedItem().toString()); 
			for (AutoCheckFrequency f : AutoCheckFrequency.values()) {
				if (f.toString().equals(sel)) {
					frequency = f;
					break;
				}
			}
			saveSettings();
			dialog.dispose();
        }
	});
	panel.add(button);
	
	button = new JButton("Cancel");
	button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			dialog.dispose();
        }
	});
	panel.add(button);
	
	button = new JButton("Check now");
	final MutableInt needToCheckNow = new MutableInt(0);
	button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			dialog.dispose();
			needToCheckNow.value = 1;
        }
	});
	panel.add(button);
	
	// Set dialog size and display:
	dialog.pack();
	Dimension ds = dialog.getSize();
	ds.width = ds.width + 10;
	ds.height = ds.height + 10; 
	Dimension scr = Toolkit.getDefaultToolkit().getScreenSize();
	dialog.setBounds((scr.width - ds.width) / 2, (scr.height - ds.height) / 2, ds.width, ds.height);
	dialog.setVisible(true);
	
	if (1 == needToCheckNow.value)
		checkForUpdates(false);
}

private void loadSettings() {
	
	GUIManager gui = LiveGraph.application().guiManager();
	
	if (!LiveGraph.application().standalone()) {
		gui.logInfoLn("Will not try loading auto-update options as LiveGraph is not"
					+ " running in stand-alone mode. Will use default settings.");
		return;
	}
	
	String fn = "";
	try {
		fn = FileTools.concatDirFile(System.getProperty("user.dir"), upgradeOptionsFileName);
	} catch(SecurityException e) {
		try {
			fn = FileTools.concatDirFile(System.getProperty("user.home"), upgradeOptionsFileName);
		} catch(SecurityException ew) {
			// If we have so few permissions, we should not try to read the update settings:
			return;
		}
	}
	
	File file = new File(fn);
	
	if (!file.exists() || !file.isFile()) {
		gui.logInfoLn("Auto-update options file not found. Will use default settings. (" + fn + ")");
		frequency = DEFAULT;
		lastCheckTS = 0L;
		return;
	}
	
	Properties props = new Properties();
	try {
		FileInputStream ins = new FileInputStream(fn);
		try {
			props.loadFromXML(ins);
		} finally {
			ins.close();
		}
	} catch(IOException e) {
		gui.logErrorLn("Error while loading auto-update options from \"" + fn + "\": " + e.getMessage());
		frequency = DEFAULT;
		lastCheckTS = 0L;
		return;
	}

	lastCheckTS = Long.parseLong(props.getProperty("lastCheckTS", "0"));
	frequency = DEFAULT;
	for (AutoCheckFrequency v : AutoCheckFrequency.values()) {
		if (v.name().equals(props.getProperty("frequency", "DEFAULT"))) {
			frequency = v;
			break;
		}
	}
	
	gui.logSuccessLn("Auto-update options loaded from \"" + fn + "\".");
}

private void saveSettings() {
	
	GUIManager gui = LiveGraph.application().guiManager();
	
	if (!LiveGraph.application().standalone()) {
		gui.logInfoLn("Will not try saving auto-update options as LiveGraph is not"
					+ " running in stand-alone mode.");
		return;
	}
	
	String fn = "";
	try {
		fn = FileTools.concatDirFile(System.getProperty("user.dir"), upgradeOptionsFileName);
	} catch(SecurityException e) {
		try {
			fn = FileTools.concatDirFile(System.getProperty("user.home"), upgradeOptionsFileName);
		} catch(SecurityException ew) {
			// If we have so few permissions, we should not try to write the update settings:
			return;
		}
	}
	
	Properties props = new Properties();
	props.setProperty("lastCheckTS", Long.toString(lastCheckTS));
	props.setProperty("frequency", frequency.name());
	
	try {
		FileOutputStream outs = new FileOutputStream(fn);
		try {
			props.storeToXML(outs,
							 "LiveGraph version " + LiveGraph.version + ". Automatic software update settings.");
		} finally {
			outs.close();
		}
		gui.logSuccessLn("Auto-update options saved to \"" + fn + "\".");
	} catch(IOException e) {
		gui.logErrorLn("Error while saving auto-update options to \"" + fn + "\": " + e.getMessage());
		return;
	}
}

} // public class UpgradeManager
