package org.LiveGraph.dataFile.write;

import java.io.File;


/**
 * Is thrown by {@code DataStreamWriterFactory} to indicate that a {@code DataStreamWriter}
 * for a specified physical file stream cannot be created becasue the file already
 * exists and cannot be overwritten. This class subclasses {@code RuntimeException} and
 * not {@code Exception} in order to allow applications the convenience of not catching
 * this exception if they have a good enough reason to believe that the data file does
 * not already exist.
 * 
 * <p>
 *   <strong>LiveGraph</strong>
 *   (<a href="http://www.live-graph.org" target="_blank">http://www.live-graph.org</a>).
 * </p> 
 * <p>Copyright (c) 2007-2008 by G. Paperin.</p>
 * <p>File: DataFileAlreadyExistsException.java</p>
 * <p style="font-size:smaller;">Redistribution and use in source and binary forms, with or
 *    without modification, are permitted provided that the following terms and conditions are met:
 * </p>
 * <p style="font-size:smaller;">1. Redistributions of source code must retain the above
 *    acknowledgement of the LiveGraph project and its web-site, the above copyright notice,
 *    this list of conditions and the following disclaimer.<br />
 *    2. Redistributions in binary form must reproduce the above acknowledgement of the
 *    LiveGraph project and its web-site, the above copyright notice, this list of conditions
 *    and the following disclaimer in the documentation and/or other materials provided with
 *    the distribution.<br />
 *    3. All advertising materials mentioning features or use of this software or any derived
 *    software must display the following acknowledgement:<br />
 *    <em>This product includes software developed by the LiveGraph project and its
 *    contributors.<br />(http://www.live-graph.org)</em><br />
 *    4. All advertising materials distributed in form of HTML pages or any other technology
 *    permitting active hyper-links that mention features or use of this software or any
 *    derived software must display the acknowledgment specified in condition 3 of this
 *    agreement, and in addition, include a visible and working hyper-link to the LiveGraph
 *    homepage (http://www.live-graph.org).
 * </p>
 * <p style="font-size:smaller;">THIS SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY
 *    OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND  NONINFRINGEMENT. IN NO EVENT SHALL
 *    THE AUTHORS, CONTRIBUTORS OR COPYRIGHT  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING  FROM, OUT OF OR
 *    IN CONNECTION WITH THE SOFTWARE OR THE USE OR  OTHER DEALINGS IN THE SOFTWARE.
 * </p>
 * 
 * @author Greg Paperin (<a href="http://www.paperin.org" target="_blank">http://www.paperin.org</a>)
 * @version {@value org.LiveGraph.LiveGraph#version}
 *
 */
public class DataFileAlreadyExistsException extends RuntimeException {


/**
 * Creates a new {@code DataFileAlreadyExistsException} relating to the specified file path.
 * 
 * @param filePath A file path.
 */
public DataFileAlreadyExistsException(String filePath) {
	super("Cannot overwrite file: " + filePath);
}

/**
 * Creates a new {@code DataFileAlreadyExistsException} relating to the specified file path.
 * 
 * @param file A file.
 */
public DataFileAlreadyExistsException(File file) {
	super("Cannot overwrite file: " + file.getAbsolutePath());
}

} // public class DataFileAlreadyExistsException
